@@ -1,5 +1,5 @@
 //
-// $Id: File.cc 1.5 Fri, 05 Nov 1999 12:07:47 -0800 yuri $
+// $Id: File.cc 1.7 Tue, 07 Dec 1999 12:14:47 -0800 yuri $
 //
 // system.cc
 // Author: Ramesh Govindan <govindan@isi.edu>
@@ -221,9 +221,16 @@ File::write(char *buffer,
     return retval;
 }
 
+Socket::Socket(int ds,FileMode fm, const Handler &rh, const Handler &wh) :
+	File(ds, fm, rh, wh) {
+}
+int Socket::setopt(int level, int optname, const void *optval, size_t optlen) {
+	return ::setsockopt(descriptor_, level, optname, optval, (SOCKLEN_T) optlen);
+}
+
 DatagramSocket::DatagramSocket(const Handler& rh,
                                const Handler& wh)
-	: File(::socket(AF_INET, SOCK_DGRAM, 0),
+	: Socket(::socket(AF_INET, SOCK_DGRAM, 0),
                FileModeReadWrite,
                rh,
                wh)
@@ -318,12 +325,14 @@ DatagramSocket::sendTo(char* buffer,
 
 int
 DatagramSocket::defaultInterface(Address& localAddress, const Address &dst) {
+#define ALLOW_MULTIPLE_IFACES
 #ifndef ALLOW_MULTIPLE_IFACES
 	static bool hashed= false;
 	static Address myOnlyIface;
-	if (hashed) 
+	if (hashed) {
 		localAddress.set(myOnlyIface.get());
-	return FileOpOK;
+		return FileOpOK;
+	}
 #endif
     int sock;
     struct sockaddr_in sin;
@@ -331,7 +340,7 @@ DatagramSocket::defaultInterface(Address& localAddress, const Address &dst) {
 
     sin.sin_family = AF_INET;
     sin.sin_addr.s_addr = htonl(dst.get());
-    //xxx: sin.sin_port = htons(port);		// We can then use write
+    sin.sin_port = htons(2000);		// We can then use write
     
     if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
         ERROR("couldn't open socket for getting local address: %s\n",
@@ -393,7 +402,7 @@ DatagramSocket::defaultInterface(Address& localAddress, const Address &dst) {
 RawSocket::RawSocket(Handler& rh,
                      Handler& wh,
                      RawSocketType type)
-	: File(::socket(AF_INET, SOCK_RAW, type),
+	: Socket(::socket(AF_INET, SOCK_RAW, type),
                FileModeReadWrite,
                rh,
                wh)
@@ -402,16 +411,16 @@ RawSocket::RawSocket(Handler& rh,
     int bval = 1;
 
 #ifdef	IP_HDRINCL
-    if(setsockopt(descriptor_, IPPROTO_IP, IP_HDRINCL,
-		  (char *)&bval, sizeof(bval)) < 0) {
-        FATAL("setsockopt IP_HDRINCL %u", bval);
-        // Not Reached
+    if(setopt(IPPROTO_IP, IP_HDRINCL,
+	      (char *)&bval, sizeof(bval)) < 0) {
+	    FATAL("setsockopt IP_HDRINCL %u", bval);
+	    // Not Reached
     }
 #endif
-    if(setsockopt(descriptor_, SOL_SOCKET, SO_RCVBUF, (char *)&bufSize,
-		  sizeof(bufSize)) < 0) {
-        FATAL("setsockopt SO_RCVBUF %u", bufSize);
-        // Not Reached
+    if(setopt(SOL_SOCKET, SO_RCVBUF, (char *)&bufSize,
+	      sizeof(bufSize)) < 0) {
+	    FATAL("setsockopt SO_RCVBUF %u", bufSize);
+	    // Not Reached
     }
 }
 
@@ -504,7 +513,7 @@ RawSocket::sendTo(char* buffer,
 }
 
 int
-RawSocket::join(Address& group)
+RawSocket::join(const Address& group)
 {
     struct ip_mreq mreq;
 
@@ -516,15 +525,14 @@ RawSocket::join(Address& group)
           group.name(),
           descriptor_);
 
-    if (::setsockopt(descriptor_,
-                     IPPROTO_IP,
-                     IP_ADD_MEMBERSHIP,
-                     (char *) &mreq,
-                     sizeof(mreq)) < 0) {
-        FATAL("failed group join on %d: %s\n",
-              descriptor_,
-              strerror(errno));
-        // NotReached
+    if (setopt(IPPROTO_IP,
+	       IP_ADD_MEMBERSHIP,
+	       (char *) &mreq,
+	       sizeof(mreq)) < 0) {
+	    FATAL("failed group join on %d: %s\n",
+		  descriptor_,
+		  strerror(errno));
+	    // NotReached
     }
 
     return FileOpOK;
@@ -568,28 +576,26 @@ MulticastSocket::MulticastSocket(const Address& addr,
           group.name(),
           descriptor_);
 
-    if (::setsockopt(descriptor_,
-                     IPPROTO_IP,
-                     IP_ADD_MEMBERSHIP,
-                     (char *) &mreq,
-                     sizeof(mreq)) < 0) {
-        FATAL("failed group join on %d: %s\n",
-              descriptor_,
-              strerror(errno));
-        // NotReached
+    if (setopt(IPPROTO_IP,
+	       IP_ADD_MEMBERSHIP,
+	       (char *) &mreq,
+	       sizeof(mreq)) < 0) {
+	    FATAL("failed group join on %d: %s\n",
+		  descriptor_,
+		  strerror(errno));
+	    // NotReached
     }
 
     // Disable loopback of packets
     loop = 0;
-    if (::setsockopt(descriptor_,
-                     IPPROTO_IP,
-                     IP_MULTICAST_LOOP,
-                     &loop,
-                     sizeof(loop)) < 0) {
-        FATAL("unable to disable multicast loopback on %d: %s\n",
-              descriptor_,
-              strerror(errno));
-        // NotReached
+    if (setopt(IPPROTO_IP,
+	       IP_MULTICAST_LOOP,
+	       &loop,
+	       sizeof(loop)) < 0) {
+	    FATAL("unable to disable multicast loopback on %d: %s\n",
+		  descriptor_,
+		  strerror(errno));
+	    // NotReached
     }
 }
 
@@ -605,11 +611,10 @@ MulticastSocket::~MulticastSocket()
           group.name(),
           descriptor_);
     
-    (void) ::setsockopt(descriptor_,
-                        IPPROTO_IP,
-                        IP_DROP_MEMBERSHIP,
-                        (char *) &mreq,
-                        sizeof(mreq));
+    setopt(IPPROTO_IP,
+	   IP_DROP_MEMBERSHIP,
+	   (char *) &mreq,
+	   sizeof(mreq));
     return;
 }
 
@@ -688,15 +693,14 @@ MulticastSocket::setTTL(int ttl)
           ttl,
           descriptor_);
     
-    if (::setsockopt(descriptor_,
-                     IPPROTO_IP,
-                     IP_MULTICAST_TTL,
-                     (char *) &t,
-                     sizeof(t)) < 0) {
-        ERROR("failed ttl set on %d: %s\n",
-              descriptor_,
-              strerror(errno));
-        return FileOpHardError;
+    if (setopt(IPPROTO_IP,
+	       IP_MULTICAST_TTL,
+	       (char *) &t,
+	       sizeof(t)) < 0) {
+	    ERROR("failed ttl set on %d: %s\n",
+		  descriptor_,
+		  strerror(errno));
+	    return FileOpHardError;
     }
     return FileOpOK;
 }
@@ -705,7 +709,7 @@ StreamSocket::StreamSocket(Address& addr,
                            Port port,
                            Handler& rh,
                            Handler& wh)
-	: File(::socket(AF_INET, SOCK_STREAM, 0),
+	: Socket(::socket(AF_INET, SOCK_STREAM, 0),
                FileModeReadWrite, 
                rh, 
                wh)
@@ -735,7 +739,7 @@ StreamSocket::StreamSocket(Address& addr,
 StreamSocket::StreamSocket(int d,
                            Handler& rh,
                            Handler& wh)
-	: File(d, FileModeReadWrite, rh, wh)
+	: Socket(d, FileModeReadWrite, rh, wh)
 {
     // Only stream sockets are non-blocking
     if (fcntl(descriptor_,
@@ -755,7 +759,7 @@ StreamSocket::~StreamSocket()
 ListenSocket::ListenSocket(Port port,
                            Handler& lh,
                            Boolean local)
-	: File(::socket(AF_INET, SOCK_STREAM, 0),
+	: Socket(::socket(AF_INET, SOCK_STREAM, 0),
                FileModeReadWrite,
                lh,
                nullHandler)
@@ -778,14 +782,13 @@ ListenSocket::ListenSocket(Port port,
           port);
 
     reuse = 1;
-    if (::setsockopt(descriptor_,
-                     SOL_SOCKET,
-                     SO_REUSEADDR,
-                     (char*) &reuse,
-                     sizeof(reuse)) < 0) {
-        FATAL("failed to set %d to be reusable: %s\n",
-              descriptor_,
-              strerror(errno));
+    if (setopt(SOL_SOCKET,
+	       SO_REUSEADDR,
+	       (char*) &reuse,
+	       sizeof(reuse)) < 0) {
+	    FATAL("failed to set %d to be reusable: %s\n",
+		  descriptor_,
+		  strerror(errno));
         // NotReached
     }
 
@@ -1069,6 +1072,7 @@ closePidFile()
             || unlink(pidFileName) == -1) {
             ERROR("could not close or remove %s: %s\n",
                   pidFileName, strerror(errno));
+	    delete [] pidFileName;
             exit(2);
         }
         pidDescriptor = -1;
